#!/usr/bin/perl

use strict;
use warnings;

sub termineEnCours
{
	my $description = shift;
	my $listeAPuce = shift;
	my $enumeration = shift;
	my $fichier = shift;
	my %dictionnaire = @_;

	if ($description && $dictionnaire{"descriptionEnCours"})
	{
		print $fichier "</dl>\n";
		$dictionnaire{"descriptionEnCours"} = 0;
	}
	if ($listeAPuce && $dictionnaire{"listeAPuceEnCours"})
	{
		print $fichier "</ul>\n";
		$dictionnaire{"listeAPuceEnCours"} = 0;
	}
	if ($enumeration && $dictionnaire{"enumerationEnCours"})
	{
		print $fichier "</ol>\n";
		$dictionnaire{"enumerationEnCours"} = 0;
	}
	return %dictionnaire;
}

sub gereStructure
{
	my $fichier = shift;
	my %dictionnaire = @_;
	return termineEnCours(1, 1, 1, $fichier, %dictionnaire);
}

open(my $entree, "<", $ARGV[0]) or die "Impossible d'ouvrir $ARGV[0]";

my $nomFichier = $ARGV[0];
$nomFichier =~ s/\.md$/\.html/;

open(my $fichier, ">", $nomFichier);
close($fichier);
open($fichier, ">>", $nomFichier);

my %dictionnaire = (descriptionEnCours => 0, listeAPuceEnCours => 0, enumerationEnCours => 0);

while (<$entree>)
{
	chomp($_);

	# Traitement de &
	$_ =~ s/&/&amp;/g;

	# Traitement de <
	$_ =~ s/</&lt;/g;

	# Traitement des espaces insécables
	$_ =~ s/ ([:;\?!])/&nbsp;$1/g;

	# Traitement des lignes horizontales
	$_ =~ s/^(\* *){3,}$/<hr>/g;

	# Mise en emphase à partir de Markdown
	$_ =~ s/(\*{2})(.+?)\*{2}/<b>$2<\/b>/g;
	$_ =~ s/(_{2})(.+?)_{2}/<i>$2<\/i>/g;

	# Traitement des notes de bas de page par [^][...]
	$_ =~ s/\[\^\]\[(.*)\]/<span class="referenceNote"><span class="note">$1<\/span><\/span>/g;

	# Traitement des images par [.jpg][...]
	$_ =~ s/\[(.*\.jpg)\]\[(.*)\]/<img src="$1" alt="$2"\/><span class="figure">$2<\/span>/g;

	my $interpretationLigne = "";

	# Titres
	if (/^(#+)([^#]+)(#*)$/)
	{
		%dictionnaire = gereStructure($fichier, %dictionnaire);
		my $nombreDeDieses = length $1;
		$interpretationLigne .= "<h".$nombreDeDieses.">".$2."</h".$nombreDeDieses.">";
		$dictionnaire{"descriptionEnCours"} = 0;
		$dictionnaire{"listeAPuceEnCours"} = 0;
		$dictionnaire{"enumerationEnCours"} = 0;
	}
	# Dictionnaires/descriptions
	#if (/^[\+\-\*]([^:]*([\[\(\{][^:]+:?[^:]+[\]\)\}])*[^:]*|[^:]+):(.+)/) {
	elsif (/^[\+\-\*] *(.+):(.+)/)
	{
		my $item = $1;
		my $definition = $2;
		chomp($item);
		chomp($definition);
		%dictionnaire = termineEnCours(0, 1, 1, $fichier, %dictionnaire);
		unless ($dictionnaire{"descriptionEnCours"})
		{
			$interpretationLigne .= "<dl>\n";
		}
		$dictionnaire{"descriptionEnCours"} = 1;
		$interpretationLigne .= "\t<div class='sansSautDePage'><dt><b>".$item.":</b></dt><dd>".$definition."</dd></div>";
	}
	# Listes simples
	elsif (/^[\+\-\*] *([^\+\-\*][^:]*)$/)
	{
		%dictionnaire = termineEnCours(1, 0, 1, $fichier, %dictionnaire);
		unless ($dictionnaire{"listeAPuceEnCours"})
		{
			$interpretationLigne .= "<ul>\n";
		}
		$dictionnaire{"listeAPuceEnCours"} = 1;
		$interpretationLigne .= "\t<li>".$1."</li>";
	}
	# Listes numérotées
	#/^[0-9]+\..+[;,.]$/ {
	elsif (/^[0-9]+\. *(.+)/)
	{
		%dictionnaire = termineEnCours(1, 1, 0, $fichier, %dictionnaire);
		unless ($dictionnaire{"enumerationEnCours"})
		{
			$interpretationLigne .= "<ol>\n";
		}
		$dictionnaire{"enumerationEnCours"} = 1;
		$interpretationLigne .= "\t<li>".$1."</li>";
	}
	else
	{
		%dictionnaire = gereStructure($fichier, %dictionnaire);
		if (length $_ > 0)
		{
			$interpretationLigne .= "<p>".$_."</p>";
		}
	}

	if (length $interpretationLigne > 0)
	{
		print $fichier "$interpretationLigne\n";
	}
}

%dictionnaire = gereStructure($fichier, %dictionnaire);

close($entree);
close($fichier);

