#!/usr/bin/perl

use strict;
use warnings;

sub termineEnCours
{
	my $description = shift;
	my $listeAPuce = shift;
	my $enumeration = shift;
	my $fichier = shift;
	my %dictionnaire = @_;

	if ($description && $dictionnaire{"descriptionEnCours"})
	{
		print $fichier "\\end{description}\n";
		$dictionnaire{"descriptionEnCours"} = 0;
	}
	if ($listeAPuce && $dictionnaire{"listeAPuceEnCours"})
	{
		print $fichier "\\end{itemize}\n";
		$dictionnaire{"listeAPuceEnCours"} = 0;
	}
	if ($enumeration && $dictionnaire{"enumerationEnCours"})
	{
		print $fichier "\\end{enumerate}\n";
		$dictionnaire{"enumerationEnCours"} = 0;
	}
	return %dictionnaire;
}

sub gereStructure
{
	my $fichier = shift;
	my %dictionnaire = @_;
	return termineEnCours(1, 1, 1, $fichier, %dictionnaire);
}

open(my $entree, "<", $ARGV[0]) or die "Impossible d'ouvrir $ARGV[0]";

my $nomFichier = $ARGV[0];
$nomFichier =~ s/\.md$/\.tex/;

open(my $fichier, ">", $nomFichier);
close($fichier);
open($fichier, ">>", $nomFichier);

my %dictionnaire = (descriptionEnCours => 0, listeAPuceEnCours => 0, enumerationEnCours => 0, paragrapheEnCours => 0);

while (<$entree>)
{
	chomp($_);

	if (/\[\]\[.*\]/)
	{
		# Traitement de code non modifié par [][...]
		$_ =~ s/\[\]\[(.*)\]/\\\[$1\\\]/g;
	}
	else
	{
		# Traitement des anti-slash (sauf devant $, ., [ et ])
		$_ =~ s/\\([^\.\$\[\]])/\\textbackslash $1/g;

		# Traitement de &, {, } et %
		$_ =~ s/([\{\}%&])/\\$1/g;

		# Traitement de «
		$_ =~ s/«/\\og /g;

		# Traitement de »
		$_ =~ s/»/~\\fg\{\}/g;

		# Traitement de $ seul sur une ligne (sinon considéré comme une formule en ligne)
		$_ =~ s/^([^\$]*)\$([^\$]*)$/$1\\\$$2/g;

		# Traitement de # en dehors d'un titre Markdown
		$_ =~ s/^([^#]+#*)#/$1\\#/g;

		# Traitement des lignes horizontales
		$_ =~ s/^(\* *){3,}$/\\par\\hrulefill/g;

		# Mise en emphase à partir de Markdown ** pour un mot-clé et __ pour un terme
		$_ =~ s/(\*{2})(.+?)\*{2}/\\motcle\{$2\}/g;
		$_ =~ s/(_{2})(.+?)_{2}/\\terme\{$2\}/g;

		# Traitement des notes de bas de page par [^][...]
		$_ =~ s/\[\^\]\[(.*)\]/\\footnote\{$1\}/g;

		# Traitement des images par [.jpg][...]
		$_ =~ s/\[(.*\.jpg)\]\[(.*)\]/\\includegraphics\[$2\]\{$1\}/g;

		# Traitement de ~ et ^
		$_ =~ s/([~^])/\\$1\{\}/g;

		# Traitement de _
		$_ =~ s/_/\\_/g;
	}


	my $interpretationLigne = "";

	# Titres
	if (/^(#+)([^#]+)(#*)$/)
	{
		%dictionnaire = gereStructure($fichier, %dictionnaire);
		my $nombreDeDieses = length $1;
		if ($nombreDeDieses == 1)
		{	# Titre de chapitre
			$interpretationLigne .= "\\chapter{".$2."}\n\\thispagestyle{fancy}";
		}
		else
		{	# début titre/sous-titre/etc.
			$interpretationLigne = "\n\\";
			for (my $i = 2; $i != $nombreDeDieses; $i += 1)
			{
				$interpretationLigne .= "sub";
			}
			$interpretationLigne .= "section{".$2."}";
		}
		$dictionnaire{"descriptionEnCours"} = 0;
		$dictionnaire{"listeAPuceEnCours"} = 0;
		$dictionnaire{"enumerationEnCours"} = 0;
		$dictionnaire{"paragrapheEnCours"} = 0;
	}
	# Dictionnaires/descriptions
	#if (/^[\+\-\*]([^:]*([\[\(\{][^:]+:?[^:]+[\]\)\}])*[^:]*|[^:]+):(.+)/) {
	elsif (/^[\+\-\*] *(.+):(.+)/)
	{
		my $item = $1;
		my $definition = $2;
		chomp($item);
		chomp($definition);
		%dictionnaire = termineEnCours(0, 1, 1, $fichier, %dictionnaire);
		unless ($dictionnaire{"descriptionEnCours"})
		{
			$interpretationLigne .= "\\begin{description}\n";
		}
		$dictionnaire{"descriptionEnCours"} = 1;
		$interpretationLigne .= "\\item[{".$item.":}]".$definition;
		$dictionnaire{"paragrapheEnCours"} = 0;
	}
	# Listes simples
	elsif (/^[\+\-\*] *([^\+\-\*][^:]*)$/)
	{
		%dictionnaire = termineEnCours(1, 0, 1, $fichier, %dictionnaire);
		unless ($dictionnaire{"listeAPuceEnCours"})
		{
			$interpretationLigne .= "\\begin{itemize}\n";
		}
		$dictionnaire{"listeAPuceEnCours"} = 1;
		$interpretationLigne .= "\\item ".$1;
		$dictionnaire{"paragrapheEnCours"} = 0;
	}
	# Listes numérotées
	#/^[0-9]+\..+[;,.]$/ {
	elsif (/^[0-9]+\. *(.+)/)
	{
		%dictionnaire = termineEnCours(1, 1, 0, $fichier, %dictionnaire);
		unless ($dictionnaire{"enumerationEnCours"})
		{
			$interpretationLigne .= "\\begin{enumerate}\n";
		}
		$dictionnaire{"enumerationEnCours"} = 1;
		$interpretationLigne .= "\\item ".$1;
		$dictionnaire{"paragrapheEnCours"} = 0;
	}
	else
	{
		%dictionnaire = gereStructure($fichier, %dictionnaire);
		if (length $_ > 0)
		{
			unless ($dictionnaire{"paragrapheEnCours"})
			{
				$interpretationLigne .= "\\paragraph{}\n";
				$dictionnaire{"paragrapheEnCours"} = 1;
			}
			$interpretationLigne .= $_;
		}
		else
		{
			$dictionnaire{"paragrapheEnCours"} = 0;
		}
	}

	if (length $interpretationLigne > 0)
	{
		print $fichier "$interpretationLigne\n";
	}
}

%dictionnaire = gereStructure($fichier, %dictionnaire);

close($entree);
close($fichier);

